import os
import pydantic

from langchain.output_parsers import PydanticOutputParser
from langchain.prompts import PromptTemplate
from langchain_openai import ChatOpenAI
from openai import OpenAI

from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_handler import ProviderHandler
from supercontrast.task.task_enum import Task
from supercontrast.task.types.ocr_types import OCRRequest, OCRResponse
from supercontrast.task.types.sentiment_analysis_types import (
    SentimentAnalysisRequest,
    SentimentAnalysisResponse,
)
from supercontrast.task.types.transcription_types import (
    TranscriptionRequest,
    TranscriptionResponse,
)
from supercontrast.task.types.translation_types import (
    TranslationRequest,
    TranslationResponse,
)
from supercontrast.utils.audio import load_audio_file
from supercontrast.utils.image import (
    get_image_size,
    load_image_data,
    process_image_for_llm,
)
from supercontrast.utils.text import truncate_text

# Constants

OPENAI_MODEL_NAME = "gpt-4o"
OPENAI_SUPPORTED_TASKS = [
    # Task.OCR, # TODO: Add back in when request is fixed
    Task.SENTIMENT_ANALYSIS,
    Task.TRANSLATION,
    Task.TRANSCRIPTION,
]
# Task.SENTIMENT_ANALYSIS


class SentimentAnalysisOutput(pydantic.BaseModel):
    score: float = pydantic.Field(
        description="Sentiment score between -1 (very negative) and 1 (very positive)"
    )


SENTIMENT_ANALYSIS_PROMPT = PromptTemplate(
    template="Analyze the sentiment of the following text. Respond with 1 for positive, 0 for neutral, and -1 for negative:\n\n{text}\n\n{format_instructions}",
    input_variables=["text"],
    partial_variables={
        "format_instructions": PydanticOutputParser(
            pydantic_object=SentimentAnalysisOutput
        ).get_format_instructions()
    },
)


class OpenAISentimentAnalysis(ProviderHandler):
    def __init__(self):
        super().__init__(provider=Provider.OPENAI, task=Task.SENTIMENT_ANALYSIS)
        model = ChatOpenAI(
            **{
                "temperature": 0,
                "model_name": OPENAI_MODEL_NAME,
            }
        ).bind(response_format={"type": "json_object"})
        parser = PydanticOutputParser(pydantic_object=SentimentAnalysisOutput)
        self.generator = SENTIMENT_ANALYSIS_PROMPT | model | parser

    def request(self, request: SentimentAnalysisRequest) -> SentimentAnalysisResponse:
        result: SentimentAnalysisOutput = self.generator.invoke(
            {"text": truncate_text(request.text)}
        )
        return SentimentAnalysisResponse(score=result.score)

    def get_name(self) -> str:
        return "OpenAI - Sentiment Analysis"

    @classmethod
    def init_from_env(cls) -> "OpenAISentimentAnalysis":
        api_key = os.environ.get("OPENAI_API_KEY")
        if not api_key:
            raise ValueError("OPENAI_API_KEY environment variable is not set")
        return cls()


# Task.TRANSLATION


class TranslationOutput(pydantic.BaseModel):
    translation: str = pydantic.Field(description="Translated text")


TRANSLATION_PROMPT = PromptTemplate(
    template="Translate the following text from {src_language} to {target_language}:\n\n{text}\n\n{format_instructions}",
    input_variables=["src_language", "target_language", "text"],
    partial_variables={
        "format_instructions": PydanticOutputParser(
            pydantic_object=TranslationOutput
        ).get_format_instructions()
    },
)


class OpenAITranslate(ProviderHandler):
    def __init__(self, src_language: str, target_language: str):
        super().__init__(provider=Provider.OPENAI, task=Task.TRANSLATION)
        model = ChatOpenAI(
            **{
                "temperature": 0,
                "model_name": OPENAI_MODEL_NAME,
            }
        ).bind(response_format={"type": "json_object"})
        parser = PydanticOutputParser(pydantic_object=TranslationOutput)
        self.generator = TRANSLATION_PROMPT | model | parser
        self.src_language = src_language
        self.target_language = target_language

    def request(self, request: TranslationRequest) -> TranslationResponse:
        result: TranslationOutput = self.generator.invoke(
            {
                "src_language": self.src_language,
                "target_language": self.target_language,
                "text": truncate_text(request.text),
            }
        )
        return TranslationResponse(text=result.translation)

    def get_name(self) -> str:
        return "OpenAI - Translate"

    @classmethod
    def init_from_env(
        cls, source_language: str, target_language: str
    ) -> "OpenAITranslate":
        api_key = os.environ.get("OPENAI_API_KEY")
        if not api_key:
            raise ValueError("OPENAI_API_KEY environment variable is not set")
        return cls(source_language, target_language)


# Task.OCR


class OCROutput(pydantic.BaseModel):
    text: str = pydantic.Field(description="Extracted text from the image")


OCR_PROMPT = PromptTemplate(
    template="Return all text that is visible in the image:\n\n{image}\n\n{format_instructions}",
    input_variables=["image"],
    partial_variables={
        "format_instructions": PydanticOutputParser(
            pydantic_object=OCROutput
        ).get_format_instructions()
    },
)


class OpenAIOCR(ProviderHandler):
    def __init__(self):
        super().__init__(provider=Provider.OPENAI, task=Task.OCR)
        model = ChatOpenAI(
            **{
                "temperature": 0,
                "model_name": OPENAI_MODEL_NAME,
            }
        ).bind(response_format={"type": "json_object"})
        parser = PydanticOutputParser(pydantic_object=OCROutput)
        self.generator = OCR_PROMPT | model | parser

    def request(self, request: OCRRequest) -> OCRResponse:
        image_data = load_image_data(request.image)
        processed_image = process_image_for_llm(image_data)
        if processed_image is None:
            raise ValueError("Failed to process the image")

        width, height = get_image_size(image_data)

        result: OCROutput = self.generator.invoke(
            {"image": f"data:image/jpeg;base64,{processed_image}"}
        )

        bounding_boxes = []

        return OCRResponse(all_text=result.text, bounding_boxes=bounding_boxes)

    def get_name(self) -> str:
        return "OpenAI - OCR"

    @classmethod
    def init_from_env(cls) -> "OpenAIOCR":
        api_key = os.environ.get("OPENAI_API_KEY")
        if not api_key:
            raise ValueError("OPENAI_API_KEY environment variable is not set")
        return cls()


# Task.TRANSCRIPTION


class OpenAITranscription(ProviderHandler):
    def __init__(self, api_key: str):
        super().__init__(provider=Provider.OPENAI, task=Task.TRANSCRIPTION)
        self.client = OpenAI(api_key=api_key)

    def request(self, request: TranscriptionRequest) -> TranscriptionResponse:
        audio_file_path = load_audio_file(request.audio_file)

        with open(audio_file_path, "rb") as audio_file:
            transcript = self.client.audio.transcriptions.create(
                model="whisper-1", file=audio_file, response_format="json"
            )

        if audio_file_path != request.audio_file:
            os.unlink(audio_file_path)  # Clean up the temporary file if it was created

        return TranscriptionResponse(text=transcript.text)

    def get_name(self) -> str:
        return "OpenAI Whisper - Transcription"

    @classmethod
    def init_from_env(cls, api_key=None) -> "OpenAITranscription":
        api_key = api_key or os.environ.get("OPENAI_API_KEY")
        if not api_key:
            raise ValueError("OPENAI_API_KEY must be set")
        return cls(api_key)


# factory


def openai_provider_factory(task: Task, **config) -> ProviderHandler:
    if task not in OPENAI_SUPPORTED_TASKS:
        raise ValueError(f"Unsupported task: {task}")
    elif task == Task.SENTIMENT_ANALYSIS:
        return OpenAISentimentAnalysis.init_from_env()
    elif task == Task.TRANSLATION:
        source_language = config.get("source_language", "en")
        target_language = config.get("target_language", "es")
        return OpenAITranslate.init_from_env(source_language, target_language)
    elif task == Task.OCR:
        return OpenAIOCR.init_from_env()
    elif task == Task.TRANSCRIPTION:
        api_key = config.get("openai_api_key")
        return OpenAITranscription.init_from_env(api_key=api_key)
    else:
        raise ValueError(f"Unsupported task: {task}")



from nltk.translate.chrf_score import sentence_chrf

from supercontrast.metrics.metrics_calculator import MetricsCalculator
from supercontrast.metrics.metrics_enum import Metric


class CHRFCalculator(MetricsCalculator[str, float]):
    def __init__(self, *args, **kwargs):
        super().__init__(Metric.CHRF, *args, **kwargs)

    def calculate(self, reference: str, hypothesis: str) -> float:
        return sentence_chrf(hypothesis, [reference])

    def get_name(self) -> str:
        return "chrF Score"



import os
import requests

from pydantic import BaseModel
from typing import Dict, List

from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_handler import ProviderHandler
from supercontrast.task.task_enum import Task
from supercontrast.task.types.ocr_types import OCRBoundingBox, OCRRequest, OCRResponse
from supercontrast.utils.image import load_image_data

# Constants

API4AI_SUPPORTED_TASKS = [Task.OCR]

# Task.OCR


class Point(BaseModel):
    x: int
    y: int


class TextSegment(BaseModel):
    text: str
    score: float
    points: List[Point]


class Entity(BaseModel):
    kind: str
    name: str
    text: str


class Object(BaseModel):
    box: List[float]
    entities: List[Entity]


class EntityGroup(BaseModel):
    kind: str
    name: str
    objects: List[Object]


class Result(BaseModel):
    status: Dict[str, str]
    name: str
    md5: str
    width: int
    height: int
    entities: List[EntityGroup]


class OCRResult(BaseModel):
    results: List[Result]


class API4AIOCR(ProviderHandler):
    def __init__(self, api_key: str):
        super().__init__(provider=Provider.API4AI, task=Task.OCR)
        self.key = api_key
        self.url = "https://ocr43.p.rapidapi.com/v1/results"
        self.headers = {"X-RapidAPI-Key": self.key}

    def request(self, request: OCRRequest) -> OCRResponse:
        image_data = load_image_data(request.image)
        files = {"image": image_data}

        response = requests.post(url=self.url, files=files, headers=self.headers)

        if response.status_code != 200:
            raise Exception(f"Error: {response.status_code} - {response.text}")

        response_json = response.json()
        response_data = OCRResult(**response_json)

        all_text = ""
        bounding_boxes = []
        for result in response_data.results:
            for entity in result.entities:
                for object in entity.objects:
                    for entity in object.entities:
                        all_text += entity.text + " "
                        coordinates = [
                            (
                                int(object.box[0] * result.width),
                                int(object.box[1] * result.height),
                            ),
                            (
                                int(object.box[2] * result.width),
                                int(object.box[1] * result.height),
                            ),
                            (
                                int(object.box[2] * result.width),
                                int(object.box[3] * result.height),
                            ),
                            (
                                int(object.box[0] * result.width),
                                int(object.box[3] * result.height),
                            ),
                        ]
                        bounding_boxes.append(
                            OCRBoundingBox(text=entity.text, coordinates=coordinates)
                        )

        return OCRResponse(all_text=all_text.strip(), bounding_boxes=bounding_boxes)

    def get_name(self) -> str:
        return "API4AI OCR"

    @classmethod
    def init_from_env(cls) -> "API4AIOCR":
        api_key = os.environ.get("API4AI_API_KEY", "")
        if not api_key:
            raise ValueError("API4AI_API_KEY is not set")
        return cls(api_key)


# factory


def api4ai_provider_factory(task: Task, **config) -> ProviderHandler:
    if task not in API4AI_SUPPORTED_TASKS:
        raise ValueError(f"Unsupported task: {task}")
    elif task == Task.OCR:
        return API4AIOCR.init_from_env()
    else:
        raise ValueError(f"Unsupported task: {task}")



import time

from abc import ABC
from typing import Dict, Generic, List, Optional, Tuple, TypeVar

from supercontrast.metrics.metrics_factory import metrics_factory
from supercontrast.optimizer.optimizer_enum import Optimizer
from supercontrast.optimizer.optimizer_factory import optimizer_factory
from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_factory import provider_factory
from supercontrast.task.task_enum import Task
from supercontrast.task.task_metadata import TaskMetadata

RequestType = TypeVar("RequestType")
ResponseType = TypeVar("ResponseType")


class TaskHandler(ABC, Generic[RequestType, ResponseType]):
    def __init__(
        self,
        task: Task,
        providers: List[Provider],
        optimizer: Optional[Optimizer] = None,
        **config,
    ):
        self.task = task
        self.provider_handler_map = {
            provider: provider_factory(task=task, provider=provider, **config)
            for provider in providers
        }
        self.metrics_handler = metrics_factory(task=task)
        self.optimizer_handler = optimizer_factory(
            task=task, providers=providers, optimizer=optimizer
        )

    def request(
        self,
        body: RequestType,
        provider: Optional[Provider] = None,
        reference: Optional[ResponseType] = None,
    ) -> Tuple[ResponseType, TaskMetadata]:
        if provider is None:
            provider = self.optimizer_handler.get_provider()

        provider_handler = self.provider_handler_map[provider]
        start_time = time.time()
        response = provider_handler.request(body)
        latency = time.time() - start_time

        metadata = TaskMetadata(
            task=self.task, provider=provider, latency=latency, reference=reference
        )

        if reference is not None and self.metrics_handler is not None:
            metrics_response = self.metrics_handler.calculate_metrics(
                reference, response
            )
            metadata.metrics = metrics_response.metrics
            metadata.normalized_reference = metrics_response.normalized_reference
            metadata.normalized_prediction = metrics_response.normalized_prediction

        return response, metadata

    def evaluate(
        self, body: RequestType, reference: Optional[ResponseType] = None
    ) -> Dict[Provider, Tuple[ResponseType, TaskMetadata]]:
        responses = {}
        for provider, handler in self.provider_handler_map.items():
            try:
                start_time = time.time()
                response = handler.request(body)
                latency = time.time() - start_time

                metadata = TaskMetadata(
                    task=self.task,
                    provider=provider,
                    latency=latency,
                )

                if self.metrics_handler is not None and reference is not None:
                    try:
                        metrics_response = self.metrics_handler.calculate_metrics(
                            reference, response
                        )
                        metadata.reference = reference
                        metadata.metrics = metrics_response.metrics
                        metadata.normalized_reference = (
                            metrics_response.normalized_reference
                        )
                        metadata.normalized_prediction = (
                            metrics_response.normalized_prediction
                        )
                    except Exception as e:
                        print(
                            f"Error calculating metrics for provider {provider}: {str(e)}"
                        )

                responses[provider] = (response, metadata)
            except Exception as e:
                # Log the error or handle it as appropriate for your use case
                print(f"Error evaluating provider {provider}: {str(e)}")
        return responses



import nltk

from nltk.tokenize import word_tokenize
from nltk.translate.meteor_score import single_meteor_score

from supercontrast.metrics.metrics_calculator import MetricsCalculator
from supercontrast.metrics.metrics_enum import Metric


class METEORCalculator(MetricsCalculator[str, float]):
    def __init__(self, *args, **kwargs):
        super().__init__(Metric.METEOR, *args, **kwargs)
        nltk.download("punkt", quiet=True)
        nltk.download("wordnet", quiet=True)

    def calculate(self, reference: str, hypothesis: str) -> float:
        reference_tokens = word_tokenize(reference)
        hypothesis_tokens = word_tokenize(hypothesis)
        return single_meteor_score(reference_tokens, hypothesis_tokens)

    def get_name(self) -> str:
        return "METEOR Score"



from test_utils import print_request_response_and_metadata

from supercontrast import (
    Provider,
    SuperContrastClient,
    Task,
    TaskMetadata,
    TranslationRequest,
    TranslationResponse,
    get_supported_providers_for_task,
)

# constants

TEST_TEXT = "Hello, world! This is a test translation."
REFERENCE_TRANSLATIONS = {
    "it": "Ciao, mondo! Questo è un test di traduzione.",
    "fr": "Bonjour, le monde! Ceci est une traduction de test.",
    "de": "Hallo, Welt! Dies ist ein Test der Übersetzung.",
    "es": "Hola, mundo! Esto es una prueba de traducción.",
}

# tests


def test_translate_anthropic():
    translate_anthropic_client = SuperContrastClient(
        task=Task.TRANSLATION,
        providers=[Provider.ANTHROPIC],
        source_language="en",
        target_language="it",
    )
    request = TranslationRequest(text=TEST_TEXT)
    reference_response = TranslationResponse(text=REFERENCE_TRANSLATIONS["it"])
    response, metadata = translate_anthropic_client.request(
        request, reference=reference_response
    )

    assert response is not None
    assert isinstance(response.text, str)
    assert len(response.text) > 0
    assert response.text != request.text

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.TRANSLATION
    assert metadata.provider == Provider.ANTHROPIC
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.TRANSLATION, request, response, metadata)


def test_translate_aws():
    translate_aws_client = SuperContrastClient(
        task=Task.TRANSLATION,
        providers=[Provider.AWS],
        source_language="en",
        target_language="fr",
    )
    request = TranslationRequest(text=TEST_TEXT)
    reference_response = TranslationResponse(text=REFERENCE_TRANSLATIONS["fr"])
    response, metadata = translate_aws_client.request(
        request, reference=reference_response
    )

    assert response is not None
    assert isinstance(response.text, str)
    assert len(response.text) > 0
    assert response.text != request.text

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.TRANSLATION
    assert metadata.provider == Provider.AWS
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.TRANSLATION, request, response, metadata)


def test_translate_azure():
    translate_azure_client = SuperContrastClient(
        task=Task.TRANSLATION,
        providers=[Provider.AZURE],
        source_language="en",
        target_language="de",
    )
    request = TranslationRequest(text=TEST_TEXT)
    reference_response = TranslationResponse(text=REFERENCE_TRANSLATIONS["de"])
    response, metadata = translate_azure_client.request(
        request, reference=reference_response
    )

    assert response is not None
    assert isinstance(response.text, str)
    assert len(response.text) > 0
    assert response.text != request.text

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.TRANSLATION
    assert metadata.provider == Provider.AZURE
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.TRANSLATION, request, response, metadata)


def test_translate_gcp():
    translate_gcp_client = SuperContrastClient(
        task=Task.TRANSLATION,
        providers=[Provider.GCP],
        source_language="en",
        target_language="it",
    )
    request = TranslationRequest(text=TEST_TEXT)
    reference_response = TranslationResponse(text=REFERENCE_TRANSLATIONS["it"])
    response, metadata = translate_gcp_client.request(
        request, reference=reference_response
    )

    assert response is not None
    assert isinstance(response.text, str)
    assert len(response.text) > 0
    assert response.text != request.text

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.TRANSLATION
    assert metadata.provider == Provider.GCP
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.TRANSLATION, request, response, metadata)


def test_translate_modernmt():
    translate_modernmt_client = SuperContrastClient(
        task=Task.TRANSLATION,
        providers=[Provider.MODERNMT],
        source_language="en",
        target_language="es",
    )
    request = TranslationRequest(text=TEST_TEXT)
    reference_response = TranslationResponse(text=REFERENCE_TRANSLATIONS["es"])
    response, metadata = translate_modernmt_client.request(
        request, reference=reference_response
    )

    assert response is not None
    assert isinstance(response.text, str)
    assert len(response.text) > 0
    assert response.text != request.text

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.TRANSLATION
    assert metadata.provider == Provider.MODERNMT
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.TRANSLATION, request, response, metadata)


def test_translate_openai():
    translate_openai_client = SuperContrastClient(
        task=Task.TRANSLATION,
        providers=[Provider.OPENAI],
        source_language="en",
        target_language="es",
    )
    request = TranslationRequest(text=TEST_TEXT)
    reference_response = TranslationResponse(text=REFERENCE_TRANSLATIONS["es"])
    response, metadata = translate_openai_client.request(
        request, reference=reference_response
    )

    assert response is not None
    assert isinstance(response.text, str)
    assert len(response.text) > 0
    assert response.text != request.text

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.TRANSLATION
    assert metadata.provider == Provider.OPENAI
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.TRANSLATION, request, response, metadata)


# evaluate


def test_translate_evaluate():
    translate_client = SuperContrastClient(
        task=Task.TRANSLATION, providers=get_supported_providers_for_task(Task.TRANSLATION),
        source_language="en",
        target_language="es",
    )
    request = TranslationRequest(text=TEST_TEXT)
    reference_response = TranslationResponse(text=REFERENCE_TRANSLATIONS["es"])
    responses = translate_client.evaluate(request, reference=reference_response)

    assert responses is not None
    assert isinstance(responses, dict)
    assert all(
        isinstance(response, tuple)
        and len(response) == 2
        and all(
            isinstance(item, (TranslationResponse, TaskMetadata)) for item in response
        )
        for response in responses.values()
    )

    for _, (response, metadata) in responses.items():
        print_request_response_and_metadata(
            Task.TRANSLATION, request, response, metadata
        )



from jiwer import cer

from supercontrast.metrics.metrics_calculator import MetricsCalculator
from supercontrast.metrics.metrics_enum import Metric


class CharacterErrorRateCalculator(MetricsCalculator[str, float]):
    def __init__(self, *args, **kwargs):
        super().__init__(Metric.CER, *args, **kwargs)

    def calculate(self, reference: str, hypothesis: str) -> float:
        result = cer(reference, hypothesis)
        if isinstance(result, float):
            return result
        else:
            raise ValueError(f"Invalid result type: {type(result)}")

    def get_name(self) -> str:
        return "Character Error Rate"



from enum import Enum


class Provider(Enum):
    ANTHROPIC = "ANTHROPIC"
    API4AI = "API4AI"
    AWS = "AWS"
    AZURE = "AZURE"
    CLARIFAI = "CLARIFAI"
    GCP = "GCP"
    MODERNMT = "MODERNMT"
    OPENAI = "OPENAI"
    SENTISIGHT = "SENTISIGHT"



from jiwer import mer, wer, wil

from supercontrast.metrics.metrics_calculator import MetricsCalculator
from supercontrast.metrics.metrics_enum import Metric


class WordErrorRateCalculator(MetricsCalculator[str, float]):
    def __init__(self, *args, **kwargs):
        super().__init__(Metric.WER, *args, **kwargs)

    def calculate(self, reference: str, hypothesis: str) -> float:
        return wer(reference, hypothesis)

    def get_name(self) -> str:
        return "Word Error Rate"


class MatchErrorRateCalculator(MetricsCalculator[str, float]):
    def __init__(self, *args, **kwargs):
        super().__init__(Metric.MER, *args, **kwargs)

    def calculate(self, reference: str, hypothesis: str) -> float:
        return mer(reference, hypothesis)

    def get_name(self) -> str:
        return "Match Error Rate"


class WordInformationLostCalculator(MetricsCalculator[str, float]):
    def __init__(self, *args, **kwargs):
        super().__init__(Metric.WIL, *args, **kwargs)

    def calculate(self, reference: str, hypothesis: str) -> float:
        return wil(reference, hypothesis)

    def get_name(self) -> str:
        return "Word Information Lost"


class WordInformationPreservedCalculator(MetricsCalculator[str, float]):
    def __init__(self, *args, **kwargs):
        super().__init__(Metric.WIP, *args, **kwargs)

    def calculate(self, reference: str, hypothesis: str) -> float:
        return 1 - wil(reference, hypothesis)

    def get_name(self) -> str:
        return "Word Information Preserved"


class WordRecognitionRateCalculator(MetricsCalculator[str, float]):
    def __init__(self, *args, **kwargs):
        super().__init__(Metric.WRR, *args, **kwargs)

    def calculate(self, reference: str, hypothesis: str) -> float:
        return 1 - wer(reference, hypothesis)

    def get_name(self) -> str:
        return "Word Recognition Rate"



from pydantic import BaseModel
from typing import Any, Dict, Generic, TypeVar

from supercontrast.metrics.metrics_enum import Metric

ResponseType = TypeVar("ResponseType")


class MetricsResponse(BaseModel, Generic[ResponseType]):
    metrics: Dict[Metric, Any]
    normalized_reference: ResponseType
    normalized_prediction: ResponseType

    class Config:
        arbitrary_types_allowed = True

    def __repr__(self):
        return (
            f"MetricsResponse(metrics={self.metrics}, "
            f"normalized_reference={self.normalized_reference}, "
            f"normalized_prediction={self.normalized_prediction})"
        )



import logging
import time

from functools import wraps
from typing import Any, Callable


def track_latency(func: Callable[..., Any]) -> Callable[..., Any]:
    @wraps(func)
    def wrapper(*args, **kwargs):
        start_time = time.time()
        result = func(*args, **kwargs)
        end_time = time.time()
        latency = end_time - start_time
        logging.info(
            f"Function '{func.__name__}' took {latency:.4f} seconds to execute."
        )
        return result

    return wrapper



import os
import requests

from io import BytesIO
from PIL import Image
from pydantic import BaseModel, Field
from typing import List

from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_handler import ProviderHandler
from supercontrast.task.task_enum import Task
from supercontrast.task.types.ocr_types import OCRBoundingBox, OCRRequest, OCRResponse
from supercontrast.utils.image import get_image_size, load_image_data

# Constants

CLARIFAI_SUPPORTED_TASKS = [
    # Task.OCR, # TODO: Add back in when request is fixed
]

# Task.OCR


class TextInfo(BaseModel):
    encoding: str


class Text(BaseModel):
    raw: str
    text_info: TextInfo


class BoundingBox(BaseModel):
    top_row: float
    left_col: float
    bottom_row: float
    right_col: float


class RegionInfo(BaseModel):
    bounding_box: BoundingBox


class Region(BaseModel):
    id: str
    region_info: RegionInfo
    data: dict
    value: float


class Output(BaseModel):
    id: str
    status: dict
    created_at: str
    model: dict
    input: dict
    data: dict = Field(..., alias="data")

    @property
    def regions(self) -> List[Region]:
        return [Region(**region) for region in self.data.get("regions", [])]


class ClarifaiResponse(BaseModel):
    status: dict
    outputs: List[Output]


class ClarifaiOCR(ProviderHandler):
    model_name = "ocr-scene-english-paddleocr"
    model_version = "46e99516c2d94f58baf2bcaf5a6a53a9"

    def __init__(self, api_key):
        super().__init__(provider=Provider.CLARIFAI, task=Task.OCR)
        self.api_key = api_key
        self.base_url = f"https://api.clarifai.com/v2/users/clarifai/apps/main/models/{self.model_name}/versions/{self.model_version}/outputs"

    def get_image_size(self, image):
        if isinstance(image, str):
            if image.startswith(("http://", "https://")):
                response = requests.get(image)
                img = Image.open(BytesIO(response.content))
            else:
                img = Image.open(image)
        elif isinstance(image, bytes):
            img = Image.open(BytesIO(image))
        else:
            raise ValueError("Unsupported image type")

        return img.size

    def request(self, request: OCRRequest) -> OCRResponse:
        headers = {
            "Authorization": f"Key {self.api_key}",
            "Content-Type": "application/json",
        }

        payload = {
            "inputs": [
                {
                    "data": {
                        "image": {
                            "url": (
                                request.image
                                if isinstance(request.image, str)
                                else None
                            )
                        }
                    }
                }
            ]
        }

        response = requests.post(self.base_url, headers=headers, json=payload)
        response.raise_for_status()

        result = response.json()
        parsed_response = ClarifaiResponse(**result)

        # Get image size
        image_size = get_image_size(request.image)

        text_parts = []
        bounding_boxes = []
        for output in parsed_response.outputs:
            for region in output.regions:
                if "text" in region.data and "raw" in region.data["text"]:
                    text = region.data["text"]["raw"]
                    text_parts.append(text)
                    bbox = region.region_info.bounding_box
                    coordinates = [
                        (
                            int(bbox.left_col * image_size[0]),
                            int(bbox.top_row * image_size[1]),
                        ),
                        (
                            int(bbox.right_col * image_size[0]),
                            int(bbox.top_row * image_size[1]),
                        ),
                        (
                            int(bbox.right_col * image_size[0]),
                            int(bbox.bottom_row * image_size[1]),
                        ),
                        (
                            int(bbox.left_col * image_size[0]),
                            int(bbox.bottom_row * image_size[1]),
                        ),
                    ]
                    bounding_boxes.append(
                        OCRBoundingBox(text=text, coordinates=coordinates)
                    )

        all_text = " ".join(text_parts)
        response = OCRResponse(all_text=all_text, bounding_boxes=bounding_boxes)
        return response

    def get_name(self) -> str:
        return "Clarifai - OCR"

    @classmethod
    def init_from_env(cls, api_key) -> "ClarifaiOCR":
        return cls(api_key)


# factory


def clarifai_provider_factory(task: Task, **config) -> ProviderHandler:
    api_key = os.environ.get("CLARIFAI_API_KEY")

    if api_key is None:
        raise ValueError("CLARIFAI_API_KEY is not set")

    if task not in CLARIFAI_SUPPORTED_TASKS:
        raise ValueError(f"Unsupported task: {task}")
    elif task == Task.OCR:
        return ClarifaiOCR.init_from_env(api_key)
    else:
        raise ValueError(f"Unsupported task: {task}")



from abc import ABC, abstractmethod
from typing import Generic, TypeVar

from supercontrast.metrics.metrics_enum import Metric

# metrics calculator


InputType = TypeVar("InputType")
OutputType = TypeVar("OutputType")


class MetricsCalculator(ABC, Generic[InputType, OutputType]):
    @abstractmethod
    def __init__(self, metric: Metric, *args, **kwargs):
        self.metric = metric

    @abstractmethod
    def calculate(self, reference: InputType, hypothesis: InputType) -> OutputType:
        raise NotImplementedError("MetricsCalculator must implement calculate method")

    @abstractmethod
    def get_name(self) -> str:
        raise NotImplementedError("MetricsCalculator must implement get_name method")



import os
import requests

from typing import Any, Dict, List, Optional

from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_handler import ProviderHandler
from supercontrast.task.task_enum import Task
from supercontrast.task.types.translation_types import (
    TranslationRequest,
    TranslationResponse,
)

# Constants

MODERNMT_SUPPORTED_TASKS = [Task.TRANSLATION]

# Task.TRANSLATION


class ModernMTTranslation(ProviderHandler):
    def __init__(self, api_key: str, source_language: str, target_language: str):
        super().__init__(provider=Provider.MODERNMT, task=Task.TRANSLATION)
        self.__base_url = "https://api.modernmt.com"
        self.__headers = {
            "MMT-ApiKey": api_key,
            "MMT-Platform": "modernmt-python",
            "MMT-PlatformVersion": "1.5.2",
        }
        self.source_language = source_language
        self.target_language = target_language

    def request(self, request: TranslationRequest) -> TranslationResponse:
        data = {
            "source": self.source_language,
            "target": self.target_language,
            "q": request.text,
        }

        response = self.__send("get", "/translate", data=data)

        if isinstance(response, list):
            translated_text = response[0]["translation"]
        else:
            translated_text = response["translation"]

        return TranslationResponse(text=translated_text)

    def get_name(self) -> str:
        return "ModernMT Translation"

    def __send(
        self, method: str, endpoint: str, data: Optional[Dict[str, Any]] = None
    ) -> Any:
        url = self.__base_url + endpoint

        headers = self.__headers.copy()
        headers["X-HTTP-Method-Override"] = method

        r = requests.post(url, headers=headers, json=data)

        if r.status_code != requests.codes.ok:
            raise ModernMTException(r.status_code, "TranslationError", r.text)

        return r.json()["data"]

    @classmethod
    def init_from_env(
        cls, source_language: str, target_language: str
    ) -> "ModernMTTranslation":
        api_key = os.environ.get("MODERN_MT_API_KEY")
        if not api_key:
            raise EnvironmentError("MODERN_MT_API_KEY environment variable is not set")
        return cls(api_key, source_language, target_language)


class ModernMTException(Exception):
    def __init__(self, status: int, type: str, message: str) -> None:
        super().__init__(f"({type}) {message}")
        self.status = status
        self.type = type
        self.message = message


def modernmt_provider_factory(task: Task, **config) -> ProviderHandler:
    if task not in MODERNMT_SUPPORTED_TASKS:
        raise ValueError(f"Unsupported task: {task}")
    elif task == Task.TRANSLATION:
        source_language = config.get("source_language", "en")
        target_language = config.get("target_language", "es")
        return ModernMTTranslation.init_from_env(
            source_language=source_language,
            target_language=target_language,
        )
    else:
        raise ValueError(f"Unsupported task: {task}")



import boto3

from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_handler import ProviderHandler
from supercontrast.task.task_enum import Task
from supercontrast.task.types.ocr_types import OCRBoundingBox, OCRRequest, OCRResponse
from supercontrast.task.types.sentiment_analysis_types import (
    SentimentAnalysisRequest,
    SentimentAnalysisResponse,
)
from supercontrast.task.types.translation_types import (
    TranslationRequest,
    TranslationResponse,
)
from supercontrast.utils.image import get_image_size, load_image_data
from supercontrast.utils.text import truncate_text

# Constants

AWS_SUPPORTED_TASKS = [Task.SENTIMENT_ANALYSIS, Task.TRANSLATION, Task.OCR]

# Task.SENTIMENT_ANALYSIS


class AWSSentimentAnalysis(ProviderHandler):
    def __init__(
        self, aws_access_key_id=None, aws_secret_access_key=None, aws_session_token=None
    ):
        super().__init__(provider=Provider.AWS, task=Task.SENTIMENT_ANALYSIS)
        self.client = boto3.client(
            "comprehend",
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token,
        )
        self.THRESHOLD = 0

    def request(self, request: SentimentAnalysisRequest) -> SentimentAnalysisResponse:
        response = self.client.detect_sentiment(
            Text=truncate_text(request.text), LanguageCode="en"
        )
        score = (
            response["SentimentScore"]["Positive"]
            - response["SentimentScore"]["Negative"]
        )

        return SentimentAnalysisResponse(score=score)

    def get_name(self) -> str:
        return "Aws Comprehend - Sentiment Analysis"

    @classmethod
    def init_from_env(
        cls, aws_access_key_id=None, aws_secret_access_key=None, aws_session_token=None
    ) -> "AWSSentimentAnalysis":
        return cls(aws_access_key_id, aws_secret_access_key, aws_session_token)


# Task.TRANSLATION


class AWSTranslate(ProviderHandler):
    def __init__(
        self,
        src_language: str,
        target_language: str,
        aws_access_key_id=None,
        aws_secret_access_key=None,
        aws_session_token=None,
    ):
        super().__init__(provider=Provider.AWS, task=Task.TRANSLATION)
        self.client = boto3.client(
            "translate",
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token,
        )
        self.src_language = src_language
        self.target_language = target_language

    def request(self, request: TranslationRequest) -> TranslationResponse:
        response = self.client.translate_text(
            Text=truncate_text(request.text),
            SourceLanguageCode=self.src_language,
            TargetLanguageCode=self.target_language,
        )
        translated_text = response["TranslatedText"]

        result = TranslationResponse(
            text=translated_text,
        )

        return result

    def get_name(self) -> str:
        return "AWS Translate"

    @classmethod
    def init_from_env(
        cls,
        source_language: str,
        target_language: str,
        aws_access_key_id=None,
        aws_secret_access_key=None,
        aws_session_token=None,
    ) -> "AWSTranslate":
        return cls(
            source_language,
            target_language,
            aws_access_key_id,
            aws_secret_access_key,
            aws_session_token,
        )


# Task.OCR


class AWSOCR(ProviderHandler):
    def __init__(
        self, aws_access_key_id=None, aws_secret_access_key=None, aws_session_token=None
    ):
        super().__init__(provider=Provider.AWS, task=Task.OCR)
        self.client = boto3.client(
            "textract",
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token,
        )

    def request(self, request: OCRRequest) -> OCRResponse:
        image_data = load_image_data(request.image)

        # Get image dimensions
        width, height = get_image_size(request.image)

        response = self.client.analyze_document(
            Document={"Bytes": image_data}, FeatureTypes=["FORMS", "TABLES"]
        )

        extracted_text = ""
        bounding_boxes = []
        for item in response.get("Blocks", []):
            if item["BlockType"] == "LINE":
                extracted_text += item["Text"] + "\n"
                if "Geometry" in item and "BoundingBox" in item["Geometry"]:
                    bbox = item["Geometry"]["BoundingBox"]
                    left = int(bbox["Left"] * width)
                    top = int(bbox["Top"] * height)
                    right = int((bbox["Left"] + bbox["Width"]) * width)
                    bottom = int((bbox["Top"] + bbox["Height"]) * height)

                    bounding_boxes.append(
                        OCRBoundingBox(
                            text=item["Text"],
                            coordinates=[
                                (left, top),
                                (right, top),
                                (right, bottom),
                                (left, bottom),
                            ],
                        )
                    )

        response = OCRResponse(
            all_text=extracted_text.strip(), bounding_boxes=bounding_boxes
        )
        return response

    def get_name(self) -> str:
        return "AWS Textract - OCR"

    @classmethod
    def init_from_env(
        cls, aws_access_key_id=None, aws_secret_access_key=None, aws_session_token=None
    ) -> "AWSOCR":
        return cls(aws_access_key_id, aws_secret_access_key, aws_session_token)


# factory


def aws_provider_factory(task: Task, **config) -> ProviderHandler:
    aws_access_key_id = config.get("aws_access_key_id")
    aws_secret_access_key = config.get("aws_secret_access_key")
    aws_session_token = config.get("aws_session_token")

    if task not in AWS_SUPPORTED_TASKS:
        raise ValueError(f"Unsupported task: {task}")
    elif task == Task.SENTIMENT_ANALYSIS:
        return AWSSentimentAnalysis.init_from_env(
            aws_access_key_id, aws_secret_access_key, aws_session_token
        )
    elif task == Task.TRANSLATION:
        source_language = config.get("source_language", "en")
        target_language = config.get("target_language", "es")
        return AWSTranslate.init_from_env(
            source_language=source_language,
            target_language=target_language,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token,
        )
    elif task == Task.OCR:
        return AWSOCR.init_from_env(
            aws_access_key_id, aws_secret_access_key, aws_session_token
        )
    else:
        raise ValueError(f"Unsupported task: {task}")



from typing import Any

from supercontrast import Task, TaskMetadata


def print_request_response_and_metadata(
    task: Task,
    request: Any,
    response: Any,
    metadata: TaskMetadata,
):
    print("\n", "-" * 80, "\n")
    print(f"{task.value.capitalize()} Request:")
    print(request, "\n")
    print(f"{task.value.capitalize()} Response from {metadata.provider}:")
    print(response, "\n")
    print("Metadata:")
    print(metadata, "\n")
    print("-" * 80, "\n")



from test_utils import print_request_response_and_metadata

from supercontrast import (
    Provider,
    SentimentAnalysisRequest,
    SentimentAnalysisResponse,
    SuperContrastClient,
    Task,
    TaskMetadata,
    get_supported_providers_for_task,
)

# constants

TEST_TEXT = "I love programming in Python!"


# tests


def test_sentiment_analysis_anthropic():
    sentiment_analysis_anthropic_client = SuperContrastClient(
        task=Task.SENTIMENT_ANALYSIS, providers=[Provider.ANTHROPIC]
    )
    request = SentimentAnalysisRequest(text=TEST_TEXT)
    response, metadata = sentiment_analysis_anthropic_client.request(request)

    assert response is not None
    assert isinstance(response.score, float)
    assert response.score > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.SENTIMENT_ANALYSIS
    assert metadata.provider == Provider.ANTHROPIC
    assert metadata.latency > 0

    print_request_response_and_metadata(
        Task.SENTIMENT_ANALYSIS, request, response, metadata
    )


def test_sentiment_analysis_aws():
    sentiment_analysis_aws_client = SuperContrastClient(
        task=Task.SENTIMENT_ANALYSIS, providers=[Provider.AWS]
    )
    request = SentimentAnalysisRequest(text=TEST_TEXT)
    response, metadata = sentiment_analysis_aws_client.request(request)

    assert response is not None
    assert isinstance(response.score, float)
    assert response.score > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.SENTIMENT_ANALYSIS
    assert metadata.provider == Provider.AWS
    assert metadata.latency > 0

    print_request_response_and_metadata(
        Task.SENTIMENT_ANALYSIS, request, response, metadata
    )


def test_sentiment_analysis_azure():
    sentiment_analysis_azure_client = SuperContrastClient(
        task=Task.SENTIMENT_ANALYSIS, providers=[Provider.AZURE]
    )
    request = SentimentAnalysisRequest(text=TEST_TEXT)
    response, metadata = sentiment_analysis_azure_client.request(request)

    assert response is not None
    assert isinstance(response.score, float)
    assert response.score > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.SENTIMENT_ANALYSIS
    assert metadata.provider == Provider.AZURE
    assert metadata.latency > 0

    print_request_response_and_metadata(
        Task.SENTIMENT_ANALYSIS, request, response, metadata
    )


def test_sentiment_analysis_gcp():
    sentiment_analysis_gcp_client = SuperContrastClient(
        task=Task.SENTIMENT_ANALYSIS, providers=[Provider.GCP]
    )
    request = SentimentAnalysisRequest(text=TEST_TEXT)
    response, metadata = sentiment_analysis_gcp_client.request(request)

    assert response is not None
    assert isinstance(response.score, float)
    assert response.score > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.SENTIMENT_ANALYSIS
    assert metadata.provider == Provider.GCP
    assert metadata.latency > 0

    print_request_response_and_metadata(
        Task.SENTIMENT_ANALYSIS, request, response, metadata
    )


def test_sentiment_analysis_openai():
    sentiment_analysis_openai_client = SuperContrastClient(
        task=Task.SENTIMENT_ANALYSIS, providers=[Provider.OPENAI]
    )
    request = SentimentAnalysisRequest(text=TEST_TEXT)
    response, metadata = sentiment_analysis_openai_client.request(request)

    assert response is not None
    assert isinstance(response.score, float)
    assert response.score > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.SENTIMENT_ANALYSIS
    assert metadata.provider == Provider.OPENAI
    assert metadata.latency > 0

    print_request_response_and_metadata(
        Task.SENTIMENT_ANALYSIS, request, response, metadata
    )


# evaluate


def test_sentiment_analysis_evaluate():
    sentiment_analysis_client = SuperContrastClient(
        task=Task.SENTIMENT_ANALYSIS, providers=get_supported_providers_for_task(Task.SENTIMENT_ANALYSIS)
    )
    request = SentimentAnalysisRequest(text=TEST_TEXT)
    responses = sentiment_analysis_client.evaluate(request)

    assert responses is not None
    assert isinstance(responses, dict)
    assert all(
        isinstance(response, tuple)
        and len(response) == 2
        and isinstance(response[0], SentimentAnalysisResponse)
        and isinstance(response[1], TaskMetadata)
        for response in responses.values()
    )

    for provider, (response, metadata) in responses.items():
        print_request_response_and_metadata(
            Task.SENTIMENT_ANALYSIS, request, response, metadata
        )

        assert isinstance(response.score, float)
        assert response.score > 0

        assert metadata.task == Task.SENTIMENT_ANALYSIS
        assert metadata.provider == provider
        assert metadata.latency > 0



from typing import List, Optional

from supercontrast.optimizer.optimizer_enum import Optimizer
from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_factory import get_supported_tasks_for_provider
from supercontrast.task.task_enum import Task
from supercontrast.task.task_handler import TaskHandler


def get_supported_providers_for_task(task: Task) -> List[Provider]:
    return [
        provider
        for provider in Provider
        if task in get_supported_tasks_for_provider(provider)
    ]


def task_factory(
    task: Task,
    providers: List[Provider],
    optimizer: Optional[Optimizer] = None,
    **config,
):
    return TaskHandler(task, providers, optimizer, **config)



from abc import ABC, abstractmethod
from typing import Generic, TypeVar

# generic types

RequestType = TypeVar("RequestType")
ResponseType = TypeVar("ResponseType")


class ProviderHandler(ABC, Generic[RequestType, ResponseType]):
    @abstractmethod
    def __init__(self, provider, task, *args, **kwargs):
        self.provider = provider
        self.task = task

    @abstractmethod
    def request(self, request: RequestType) -> ResponseType:
        raise NotImplementedError("ProviderHandler must implement request method")

    # @abstractmethod
    # async def request_async(self, request: RequestType) -> ResponseType:
    #     raise NotImplementedError("Subclass must implement abstract method")

    # @abstractmethod
    # def batch_request(self, requests: List[RequestType]) -> List[ResponseType]:
    #     raise NotImplementedError("Subclass must implement abstract method")

    @abstractmethod
    def get_name(self) -> str:
        raise NotImplementedError("ProviderHandler must implement get_name method")

    @classmethod
    @abstractmethod
    def init_from_env(cls, *args, **kwargs) -> "ProviderHandler":
        raise NotImplementedError("ProviderHandler must implement init_from_env method")



from supercontrast import (
    Provider,
    Task,
    get_supported_providers_for_task,
    get_supported_tasks_for_provider,
)


def test_get_supported_providers_for_task():
    assert set(get_supported_providers_for_task(Task.OCR)) == {
        Provider.API4AI,
        Provider.AWS,
        Provider.AZURE,
        Provider.GCP,
        Provider.SENTISIGHT,
    }
    assert set(get_supported_providers_for_task(Task.SENTIMENT_ANALYSIS)) == {
        Provider.ANTHROPIC,
        Provider.AWS,
        Provider.AZURE,
        Provider.GCP,
        Provider.OPENAI,
    }
    assert set(get_supported_providers_for_task(Task.TRANSCRIPTION)) == {
        Provider.AZURE,
        Provider.OPENAI,
    }
    assert set(get_supported_providers_for_task(Task.TRANSLATION)) == {
        Provider.ANTHROPIC,
        Provider.AWS,
        Provider.AZURE,
        Provider.GCP,
        Provider.MODERNMT,
        Provider.OPENAI,
    }


def test_get_supported_tasks_for_provider():
    assert set(get_supported_tasks_for_provider(Provider.ANTHROPIC)) == {
        Task.SENTIMENT_ANALYSIS,
        Task.TRANSLATION,
    }
    assert set(get_supported_tasks_for_provider(Provider.API4AI)) == {Task.OCR}
    assert set(get_supported_tasks_for_provider(Provider.AWS)) == {
        Task.OCR,
        Task.SENTIMENT_ANALYSIS,
        Task.TRANSLATION,
    }
    assert set(get_supported_tasks_for_provider(Provider.AZURE)) == {
        Task.OCR,
        Task.SENTIMENT_ANALYSIS,
        Task.TRANSCRIPTION,
        Task.TRANSLATION,
    }
    # assert set(get_supported_tasks_for_provider(Provider.CLARIFAI)) == {}
    assert set(get_supported_tasks_for_provider(Provider.GCP)) == {
        Task.OCR,
        Task.SENTIMENT_ANALYSIS,
        Task.TRANSLATION,
    }
    assert set(get_supported_tasks_for_provider(Provider.MODERNMT)) == {
        Task.TRANSLATION
    }
    assert set(get_supported_tasks_for_provider(Provider.OPENAI)) == {
        Task.SENTIMENT_ANALYSIS,
        Task.TRANSCRIPTION,
        Task.TRANSLATION,
    }
    assert set(get_supported_tasks_for_provider(Provider.SENTISIGHT)) == {Task.OCR}



from abc import ABC, abstractmethod
from typing import List

from supercontrast.optimizer.optimizer_enum import Optimizer
from supercontrast.provider.provider_enum import Provider
from supercontrast.task.task_enum import Task


class OptimizerHandler(ABC):
    def __init__(self, task: Task, providers: List[Provider]):
        self.task = task
        self.providers = providers

    @abstractmethod
    def get_provider(self) -> Provider:
        pass



import azure.cognitiveservices.speech as speechsdk
import io
import os
import time

from azure.ai.textanalytics import TextAnalyticsClient
from azure.ai.translation.text import TextTranslationClient
from azure.cognitiveservices.vision.computervision import ComputerVisionClient
from azure.cognitiveservices.vision.computervision.models import OperationStatusCodes
from azure.core.credentials import AzureKeyCredential
from msrest.authentication import CognitiveServicesCredentials

from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_handler import ProviderHandler
from supercontrast.task.task_enum import Task
from supercontrast.task.types.ocr_types import OCRBoundingBox, OCRRequest, OCRResponse
from supercontrast.task.types.sentiment_analysis_types import (
    SentimentAnalysisRequest,
    SentimentAnalysisResponse,
)
from supercontrast.task.types.transcription_types import (
    TranscriptionRequest,
    TranscriptionResponse,
)
from supercontrast.task.types.translation_types import (
    TranslationRequest,
    TranslationResponse,
)
from supercontrast.utils.audio import load_audio_file
from supercontrast.utils.image import get_image_size, load_image_data

# Constants

AZURE_SUPPORTED_TASKS = [
    Task.SENTIMENT_ANALYSIS,
    Task.TRANSLATION,
    Task.OCR,
    Task.TRANSCRIPTION,
]

# Task.SENTIMENT_ANALYSIS


class AzureSentimentAnalysis(ProviderHandler):
    def __init__(self, endpoint: str, key: str):
        super().__init__(provider=Provider.AZURE, task=Task.SENTIMENT_ANALYSIS)
        self.client = TextAnalyticsClient(endpoint, AzureKeyCredential(key))

    def request(self, request: SentimentAnalysisRequest) -> SentimentAnalysisResponse:
        response = self.client.analyze_sentiment([request.text])[0]
        score = (
            response.confidence_scores.positive - response.confidence_scores.negative
        )
        return SentimentAnalysisResponse(score=score)

    def get_name(self) -> str:
        return "Azure Text Analytics - Sentiment Analysis"

    @classmethod
    def init_from_env(cls, endpoint=None, key=None) -> "AzureSentimentAnalysis":
        endpoint = endpoint or os.environ.get("AZURE_TEXT_ANALYTICS_ENDPOINT")
        key = key or os.environ.get("AZURE_TEXT_ANALYTICS_KEY")
        if not endpoint or not key:
            raise ValueError(
                "AZURE_TEXT_ANALYTICS_ENDPOINT and AZURE_TEXT_ANALYTICS_KEY must be set"
            )

        return cls(endpoint, key)


# Task.TRANSLATION


class AzureTranslation(ProviderHandler):
    def __init__(
        self, key: str, region: str, source_language: str, target_language: str
    ):
        super().__init__(provider=Provider.AZURE, task=Task.TRANSLATION)
        self.client = TextTranslationClient(
            credential=AzureKeyCredential(key), region=region
        )
        self.source_language = source_language
        self.target_language = target_language

    def request(self, request: TranslationRequest) -> TranslationResponse:
        response = self.client.translate(
            body=[request.text],
            from_language=self.source_language,
            to_language=[self.target_language],
        )
        translated_text = response[0].translations[0].text
        return TranslationResponse(text=translated_text)

    def get_name(self) -> str:
        return "Azure Translator"

    @classmethod
    def init_from_env(
        cls, source_language: str, target_language: str, key=None, region=None
    ) -> "AzureTranslation":
        key = key or os.environ.get("AZURE_TEXT_ANALYTICS_KEY")
        region = region or os.environ.get("AZURE_TRANSLATOR_REGION")
        if not key or not region:
            raise ValueError(
                "AZURE_TEXT_ANALYTICS_KEY and AZURE_TRANSLATOR_REGION must be set"
            )

        return cls(key, region, source_language, target_language)


# Task.OCR


class AzureOCR(ProviderHandler):
    def __init__(self, endpoint: str, key: str):
        super().__init__(provider=Provider.AZURE, task=Task.OCR)
        self.client = ComputerVisionClient(endpoint, CognitiveServicesCredentials(key))

    def request(self, request: OCRRequest) -> OCRResponse:
        image_data = load_image_data(request.image)
        read_response = self.client.read_in_stream(io.BytesIO(image_data), raw=True)

        if not read_response:
            raise ValueError("Failed to read image")

        operation_location = read_response.headers["Operation-Location"]

        if not operation_location:
            raise ValueError("Failed to get operation location")

        operation_id = operation_location.split("/")[-1]

        while True:
            read_result = self.client.get_read_result(operation_id)

            if read_result.status not in ["notStarted", "running"]:  # type: ignore
                break
            time.sleep(1)

        extracted_text = ""
        bounding_boxes = []

        if read_result.status == OperationStatusCodes.succeeded:  # type: ignore
            for text_result in read_result.analyze_result.read_results:  # type: ignore
                for line in text_result.lines:
                    extracted_text += line.text + "\n"
                    bounding_boxes.append(
                        OCRBoundingBox(
                            text=line.text,
                            coordinates=[
                                (line.bounding_box[0], line.bounding_box[1]),
                                (line.bounding_box[2], line.bounding_box[3]),
                                (line.bounding_box[4], line.bounding_box[5]),
                                (line.bounding_box[6], line.bounding_box[7]),
                            ],
                        )
                    )

        response = OCRResponse(
            all_text=extracted_text.strip(), bounding_boxes=bounding_boxes
        )
        return response

    def get_name(self) -> str:
        return "Azure Computer Vision - OCR"

    @classmethod
    def init_from_env(cls, endpoint=None, key=None) -> "AzureOCR":
        endpoint = endpoint or os.environ.get("AZURE_VISION_ENDPOINT")
        key = key or os.environ.get("AZURE_VISION_KEY")
        if not endpoint or not key:
            raise ValueError("AZURE_VISION_ENDPOINT and AZURE_VISION_KEY must be set")

        return cls(endpoint, key)


# Task.TRANSCRIPTION


class AzureTranscription(ProviderHandler):
    def __init__(self, speech_key: str, service_region: str):
        super().__init__(provider=Provider.AZURE, task=Task.TRANSCRIPTION)
        self.speech_key = speech_key
        self.service_region = service_region

    def request(self, request: TranscriptionRequest) -> TranscriptionResponse:
        audio_file_path = load_audio_file(request.audio_file)

        speech_config = speechsdk.SpeechConfig(
            subscription=self.speech_key, region=self.service_region
        )
        audio_config = speechsdk.audio.AudioConfig(filename=audio_file_path)

        speech_recognizer = speechsdk.SpeechRecognizer(
            speech_config=speech_config, audio_config=audio_config
        )

        complete_transcription = []
        done = False

        def recognized_cb(evt):
            if evt.result.reason == speechsdk.ResultReason.RecognizedSpeech:
                complete_transcription.append(evt.result.text)
            elif evt.result.reason == speechsdk.ResultReason.NoMatch:
                print("No speech could be recognized")

        def stop_cb(evt):
            nonlocal done
            done = True

        speech_recognizer.recognized.connect(recognized_cb)
        speech_recognizer.session_stopped.connect(stop_cb)
        speech_recognizer.session_started.connect(lambda evt: print("Session started"))

        speech_recognizer.start_continuous_recognition()

        while not done:
            time.sleep(0.5)

        speech_recognizer.stop_continuous_recognition()

        if audio_file_path != request.audio_file:
            os.unlink(audio_file_path)

        return TranscriptionResponse(text=" ".join(complete_transcription))

    def get_name(self) -> str:
        return "Azure Speech Services - Transcription"

    @classmethod
    def init_from_env(
        cls, speech_key=None, service_region=None
    ) -> "AzureTranscription":
        speech_key = speech_key or os.environ.get("AZURE_SPEECH_KEY")
        service_region = service_region or os.environ.get("AZURE_SPEECH_REGION")
        if not speech_key or not service_region:
            raise ValueError("AZURE_SPEECH_KEY and AZURE_SPEECH_REGION must be set")
        return cls(speech_key, service_region)


# factory


def azure_provider_factory(task: Task, **config) -> ProviderHandler:
    if task not in AZURE_SUPPORTED_TASKS:
        raise ValueError(f"Unsupported task: {task}")
    elif task == Task.SENTIMENT_ANALYSIS:
        endpoint = config.get("azure_text_analytics_endpoint")
        key = config.get("azure_text_analytics_key")
        return AzureSentimentAnalysis.init_from_env(endpoint=endpoint, key=key)
    elif task == Task.TRANSLATION:
        source_language = config.get("source_language", "en")
        target_language = config.get("target_language", "es")
        key = config.get("azure_text_analytics_key")
        region = config.get("azure_translator_region")
        return AzureTranslation.init_from_env(
            source_language=source_language,
            target_language=target_language,
            key=key,
            region=region,
        )
    elif task == Task.OCR:
        endpoint = config.get("azure_vision_endpoint")
        key = config.get("azure_vision_key")
        return AzureOCR.init_from_env(endpoint=endpoint, key=key)
    elif task == Task.TRANSCRIPTION:
        speech_key = config.get("azure_speech_key")
        service_region = config.get("azure_speech_region")
        return AzureTranscription.init_from_env(
            speech_key=speech_key, service_region=service_region
        )
    else:
        raise ValueError(f"Unsupported task: {task}")



import base64
import os
import requests

from io import BytesIO
from PIL import Image
from typing import Tuple, Union


def get_image_size(image: Union[str, bytes]) -> Tuple[int, int]:
    """
    Get the size of an image.

    Args:
        image (Union[str, bytes]): The image to get the size of. Can be a URL, a local file path, or bytes.

    Returns:
        Tuple[int, int]: A tuple containing the width and height of the image.

    Raises:
        ValueError: If the image type is unsupported.
    """
    if isinstance(image, str):
        if image.startswith(("http://", "https://")):
            response = requests.get(image)
            img = Image.open(BytesIO(response.content))
        else:
            img = Image.open(image)
    elif isinstance(image, bytes):
        img = Image.open(BytesIO(image))
    else:
        raise ValueError("Unsupported image type")

    return img.size


def load_image_data(image: Union[str, bytes]) -> bytes:
    """
    Load image data from various sources.

    Args:
        image (Union[str, bytes]): The image to load. Can be a URL, a local file path, or bytes.

    Returns:
        bytes: The image data as bytes.

    Raises:
        ValueError: If the image type is unsupported.
    """
    if isinstance(image, str):
        if image.startswith(("http://", "https://")):
            return requests.get(image).content
        else:
            with open(image, "rb") as image_file:
                return image_file.read()
    elif isinstance(image, bytes):
        return image
    else:
        raise ValueError("Unsupported image type")


# Image processing for LLMs


def encode_image(image_data: bytes) -> str:
    return base64.b64encode(image_data).decode("utf-8")


def convert_to_jpeg_and_resize(image_data: bytes, max_size: int = 1024) -> bytes | None:
    try:
        with Image.open(BytesIO(image_data)) as img:
            if img.mode != "RGB":
                img = img.convert("RGB")
            original_width, original_height = img.size
            ratio = min(max_size / original_width, max_size / original_height)
            new_size = (int(original_width * ratio), int(original_height * ratio))
            img = img.resize(new_size)
            output_buffer = BytesIO()
            img.save(output_buffer, format="JPEG")
            return output_buffer.getvalue()
    except IOError:
        print("Error in converting the image to JPEG")
        return None


def process_image_for_llm(image_data: bytes) -> str | None:
    jpeg_data = convert_to_jpeg_and_resize(image_data)
    if jpeg_data is None:
        return None
    return encode_image(jpeg_data)



from typing import List

from supercontrast.provider.handlers.anthropic_handler import (
    ANTHROPIC_SUPPORTED_TASKS,
    anthropic_provider_factory,
)
from supercontrast.provider.handlers.api4ai_handler import (
    API4AI_SUPPORTED_TASKS,
    api4ai_provider_factory,
)
from supercontrast.provider.handlers.aws_handler import (
    AWS_SUPPORTED_TASKS,
    aws_provider_factory,
)
from supercontrast.provider.handlers.azure_handler import (
    AZURE_SUPPORTED_TASKS,
    azure_provider_factory,
)
from supercontrast.provider.handlers.clarifai_handler import (
    CLARIFAI_SUPPORTED_TASKS,
    clarifai_provider_factory,
)
from supercontrast.provider.handlers.gcp_handler import (
    GCP_SUPPORTED_TASKS,
    gcp_provider_factory,
)
from supercontrast.provider.handlers.modern_mt_handler import (
    MODERNMT_SUPPORTED_TASKS,
    modernmt_provider_factory,
)
from supercontrast.provider.handlers.openai_handler import (
    OPENAI_SUPPORTED_TASKS,
    openai_provider_factory,
)
from supercontrast.provider.handlers.sentisight_handler import (
    SENTISIGHT_SUPPORTED_TASKS,
    sentisight_provider_factory,
)
from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_handler import ProviderHandler
from supercontrast.task.task_enum import Task


def get_supported_tasks_for_provider(provider: Provider) -> List[Task]:
    if provider == Provider.ANTHROPIC:
        return ANTHROPIC_SUPPORTED_TASKS
    elif provider == Provider.API4AI:
        return API4AI_SUPPORTED_TASKS
    elif provider == Provider.AWS:
        return AWS_SUPPORTED_TASKS
    elif provider == Provider.AZURE:
        return AZURE_SUPPORTED_TASKS
    elif provider == Provider.CLARIFAI:
        return CLARIFAI_SUPPORTED_TASKS
    elif provider == Provider.GCP:
        return GCP_SUPPORTED_TASKS
    elif provider == Provider.MODERNMT:
        return MODERNMT_SUPPORTED_TASKS
    elif provider == Provider.OPENAI:
        return OPENAI_SUPPORTED_TASKS
    elif provider == Provider.SENTISIGHT:
        return SENTISIGHT_SUPPORTED_TASKS
    else:
        raise ValueError(f"Unsupported provider: {provider}")


def provider_factory(task: Task, provider: Provider, **config) -> ProviderHandler:
    if provider == Provider.ANTHROPIC and task in ANTHROPIC_SUPPORTED_TASKS:
        return anthropic_provider_factory(task, **config)
    elif provider == Provider.API4AI and task in API4AI_SUPPORTED_TASKS:
        return api4ai_provider_factory(task, **config)
    elif provider == Provider.AWS and task in AWS_SUPPORTED_TASKS:
        return aws_provider_factory(task, **config)
    elif provider == Provider.AZURE and task in AZURE_SUPPORTED_TASKS:
        return azure_provider_factory(task, **config)
    elif provider == Provider.CLARIFAI and task in CLARIFAI_SUPPORTED_TASKS:
        return clarifai_provider_factory(task, **config)
    elif provider == Provider.GCP and task in GCP_SUPPORTED_TASKS:
        return gcp_provider_factory(task, **config)
    elif provider == Provider.MODERNMT and task in MODERNMT_SUPPORTED_TASKS:
        return modernmt_provider_factory(task, **config)
    elif provider == Provider.OPENAI and task in OPENAI_SUPPORTED_TASKS:
        return openai_provider_factory(task, **config)
    elif provider == Provider.SENTISIGHT and task in SENTISIGHT_SUPPORTED_TASKS:
        return sentisight_provider_factory(task, **config)
    else:
        raise ValueError(f"Task {task} is not supported by provider {provider}")



import os
import requests

from pydantic import BaseModel
from typing import List

from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_handler import ProviderHandler
from supercontrast.task.task_enum import Task
from supercontrast.task.types.ocr_types import OCRBoundingBox, OCRRequest, OCRResponse
from supercontrast.utils.image import load_image_data

# Constants

SENTISIGHT_SUPPORTED_TASKS = [Task.OCR]

# Task.OCR


class Point(BaseModel):
    x: int
    y: int


class TextSegment(BaseModel):
    label: str
    score: float
    points: List[Point]


class SentisightOCRResult(BaseModel):
    segments: List[TextSegment]


class SentisightOCR(ProviderHandler):
    def __init__(self, api_key: str, language: str):
        super().__init__(provider=Provider.SENTISIGHT, task=Task.OCR)
        self.key = api_key
        self.language = language
        self.base_url = "https://platform.sentisight.ai/api/pm-predict/"

    def request(self, request: OCRRequest) -> OCRResponse:
        image_data = load_image_data(request.image)

        url = f"{self.base_url}Text-recognition"

        response = requests.post(
            url=f"{url}?lang={self.language}",
            headers={
                "accept": "*/*",
                "X-Auth-token": self.key,
                "Content-Type": "application/octet-stream",
            },
            data=image_data,
        )

        if response.status_code != 200:
            raise Exception(f"Error: {response.status_code} - {response.text}")

        response_data: SentisightOCRResult = SentisightOCRResult(
            segments=[TextSegment(**item) for item in response.json()]
        )

        text = ""
        for segment in response_data.segments:
            text += segment.label + "\n"

        response = OCRResponse(
            all_text=text,
            bounding_boxes=[
                OCRBoundingBox(
                    text=segment.label,
                    coordinates=[(point.x, point.y) for point in segment.points],
                )
                for segment in response_data.segments
            ],
        )
        return response

    def get_name(self) -> str:
        return "Sentisight OCR"

    @classmethod
    def init_from_env(cls, language: str) -> "SentisightOCR":
        api_key = os.environ.get("SENTISIGHT_API_TOKEN")
        if not api_key:
            raise EnvironmentError(
                "SENTISIGHT_API_TOKEN environment variable is not set"
            )
        return cls(api_key, language)


def sentisight_provider_factory(task: Task, **config) -> ProviderHandler:
    if task not in SENTISIGHT_SUPPORTED_TASKS:
        raise ValueError(f"Unsupported task: {task}")
    elif task == Task.OCR:
        language = config.get("language", "en")
        return SentisightOCR.init_from_env(language=language)
    else:
        raise ValueError(f"Unsupported task: {task}")



from test_utils import print_request_response_and_metadata

from supercontrast import (
    OCRRequest,
    OCRResponse,
    Provider,
    SuperContrastClient,
    Task,
    TaskMetadata,
    get_supported_providers_for_task,
)

# constants

TEST_IMAGE_URL = "https://github.com/supercontrast-ai/supercontrast/raw/main/tests/image/test_ocr.png"
REFERENCE_OCR_TEXT = "This is a lot of 12 point text to test the\nocr code and see if it works on all types\nof file format.\nThe quick brown dog jumped over the\nlazy fox. The quick brown dog jumped\nover the lazy fox. The quick brown dog\njumped over the lazy fox. The quick\nbrown dog jumped over the lazy fox."


# tests


def test_ocr_api4ai():
    ocr_api4ai_client = SuperContrastClient(task=Task.OCR, providers=[Provider.API4AI])
    request = OCRRequest(image=TEST_IMAGE_URL)
    reference_response = OCRResponse(all_text=REFERENCE_OCR_TEXT, bounding_boxes=[])
    response, metadata = ocr_api4ai_client.request(
        request, reference=reference_response
    )

    assert response is not None
    assert isinstance(response.all_text, str)
    assert len(response.all_text) > 0
    assert len(response.bounding_boxes) > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.OCR
    assert metadata.provider == Provider.API4AI
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.OCR, request, response, metadata)


def test_ocr_aws():
    ocr_aws_client = SuperContrastClient(task=Task.OCR, providers=[Provider.AWS])
    request = OCRRequest(image=TEST_IMAGE_URL)
    reference_response = OCRResponse(all_text=REFERENCE_OCR_TEXT, bounding_boxes=[])
    response, metadata = ocr_aws_client.request(request, reference=reference_response)

    assert response is not None
    assert isinstance(response.all_text, str)
    assert len(response.all_text) > 0
    assert len(response.bounding_boxes) > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.OCR
    assert metadata.provider == Provider.AWS
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.OCR, request, response, metadata)


def test_ocr_azure():
    ocr_azure_client = SuperContrastClient(task=Task.OCR, providers=[Provider.AZURE])
    request = OCRRequest(image=TEST_IMAGE_URL)
    reference_response = OCRResponse(all_text=REFERENCE_OCR_TEXT, bounding_boxes=[])
    response, metadata = ocr_azure_client.request(request, reference=reference_response)

    assert response is not None
    assert isinstance(response.all_text, str)
    assert len(response.all_text) > 0
    assert len(response.bounding_boxes) > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.OCR
    assert metadata.provider == Provider.AZURE
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.OCR, request, response, metadata)


def test_ocr_clarifai():
    ocr_clarifai_client = SuperContrastClient(
        task=Task.OCR, providers=[Provider.CLARIFAI]
    )
    request = OCRRequest(image=TEST_IMAGE_URL)
    reference_response = OCRResponse(all_text=REFERENCE_OCR_TEXT, bounding_boxes=[])
    response, metadata = ocr_clarifai_client.request(
        request, reference=reference_response
    )

    assert response is not None
    assert isinstance(response.all_text, str)
    assert len(response.all_text) > 0
    assert len(response.bounding_boxes) > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.OCR
    assert metadata.provider == Provider.CLARIFAI
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.OCR, request, response, metadata)


def test_ocr_gcp():
    ocr_gcp_client = SuperContrastClient(task=Task.OCR, providers=[Provider.GCP])
    request = OCRRequest(image=TEST_IMAGE_URL)
    reference_response = OCRResponse(all_text=REFERENCE_OCR_TEXT, bounding_boxes=[])
    response, metadata = ocr_gcp_client.request(request, reference=reference_response)

    assert response is not None
    assert isinstance(response.all_text, str)
    assert len(response.all_text) > 0
    assert len(response.bounding_boxes) > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.OCR
    assert metadata.provider == Provider.GCP
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.OCR, request, response, metadata)


def test_ocr_sentisight():
    ocr_sentisight_client = SuperContrastClient(
        task=Task.OCR, providers=[Provider.SENTISIGHT]
    )
    request = OCRRequest(image=TEST_IMAGE_URL)
    reference_response = OCRResponse(all_text=REFERENCE_OCR_TEXT, bounding_boxes=[])
    response, metadata = ocr_sentisight_client.request(
        request, reference=reference_response
    )

    assert response is not None
    assert isinstance(response.all_text, str)
    assert len(response.all_text) > 0
    assert len(response.bounding_boxes) > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.OCR
    assert metadata.provider == Provider.SENTISIGHT
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.OCR, request, response, metadata)


# evaluate


def test_ocr_evaluate():
    ocr_client = SuperContrastClient(
        task=Task.OCR, providers=get_supported_providers_for_task(Task.OCR)
    )
    request = OCRRequest(image=TEST_IMAGE_URL)
    reference_response = OCRResponse(all_text=REFERENCE_OCR_TEXT, bounding_boxes=[])
    responses = ocr_client.evaluate(request, reference=reference_response)

    assert responses is not None
    assert isinstance(responses, dict)
    assert all(
        isinstance(response, tuple)
        and len(response) == 2
        and isinstance(response[0], OCRResponse)
        and isinstance(response[1], TaskMetadata)
        for response in responses.values()
    )

    for provider, (response, metadata) in responses.items():
        print_request_response_and_metadata(Task.OCR, request, response, metadata)

        assert isinstance(response.all_text, str)
        assert len(response.all_text) > 0
        assert len(response.bounding_boxes) > 0

        assert metadata.task == Task.OCR
        assert metadata.provider == provider
        assert metadata.latency > 0



import os

from google.cloud import language_v1, translate_v2, vision_v1
from google.oauth2 import service_account

from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_handler import ProviderHandler
from supercontrast.task.task_enum import Task
from supercontrast.task.types.ocr_types import OCRBoundingBox, OCRRequest, OCRResponse
from supercontrast.task.types.sentiment_analysis_types import (
    SentimentAnalysisRequest,
    SentimentAnalysisResponse,
)
from supercontrast.task.types.translation_types import (
    TranslationRequest,
    TranslationResponse,
)
from supercontrast.utils.image import load_image_data

# Constants

GCP_SUPPORTED_TASKS = [Task.SENTIMENT_ANALYSIS, Task.TRANSLATION, Task.OCR]

# Task.SENTIMENT_ANALYSIS


class GCPSentimentAnalysis(ProviderHandler):
    def __init__(self, credentials):
        super().__init__(provider=Provider.GCP, task=Task.SENTIMENT_ANALYSIS)
        self.client = language_v1.LanguageServiceClient(credentials=credentials)
        self.THRESHOLD = 0

    def request(self, request: SentimentAnalysisRequest) -> SentimentAnalysisResponse:
        document = language_v1.Document(
            content=request.text, type_=language_v1.Document.Type.PLAIN_TEXT
        )
        sentiment = self.client.analyze_sentiment(
            request={"document": document}
        ).document_sentiment

        score = sentiment.score

        return SentimentAnalysisResponse(score=score)

    def get_name(self) -> str:
        return "Google Natural Language - Sentiment Analysis"

    @classmethod
    def init_from_env(cls) -> "GCPSentimentAnalysis":
        credentials_path = os.environ.get("GOOGLE_APPLICATION_CREDENTIALS")
        if not credentials_path:
            raise EnvironmentError(
                "GOOGLE_APPLICATION_CREDENTIALS environment variable is not set"
            )
        credentials = service_account.Credentials.from_service_account_file(
            credentials_path
        )
        return cls(credentials)


# Task.TRANSLATION


class GCPTranslation(ProviderHandler):
    def __init__(self, credentials, src_language: str, target_language: str):
        super().__init__(provider=Provider.GCP, task=Task.TRANSLATION)
        self.client = translate_v2.Client(credentials=credentials)
        self.src_language = src_language
        self.target_language = target_language

    def request(self, request: TranslationRequest) -> TranslationResponse:
        result = self.client.translate(
            request.text,
            source_language=self.src_language,
            target_language=self.target_language,
        )
        translated_text = result["translatedText"]

        return TranslationResponse(text=translated_text)

    def get_name(self) -> str:
        return "Google Translation"

    @classmethod
    def init_from_env(
        cls, source_language: str, target_language: str
    ) -> "GCPTranslation":
        credentials_path = os.environ.get("GOOGLE_APPLICATION_CREDENTIALS")
        if not credentials_path:
            raise EnvironmentError(
                "GOOGLE_APPLICATION_CREDENTIALS environment variable is not set"
            )
        credentials = service_account.Credentials.from_service_account_file(
            credentials_path
        )
        return cls(credentials, source_language, target_language)


# Task.OCR


class GCPOCR(ProviderHandler):
    def __init__(self, credentials):
        super().__init__(provider=Provider.GCP, task=Task.OCR)
        self.client = vision_v1.ImageAnnotatorClient(credentials=credentials)

    def request(self, request: OCRRequest) -> OCRResponse:
        image_data = load_image_data(request.image)
        image = vision_v1.Image(content=image_data)

        response = self.client.document_text_detection(image=image)  # type: ignore

        extracted_text = response.full_text_annotation.text  # type: ignore

        bounding_boxes = []
        for page in response.full_text_annotation.pages:
            for block in page.blocks:
                for paragraph in block.paragraphs:
                    for word in paragraph.words:
                        word_text = "".join([symbol.text for symbol in word.symbols])
                        vertices = [
                            (vertex.x, vertex.y)
                            for vertex in word.bounding_box.vertices
                        ]
                        bounding_boxes.append(
                            OCRBoundingBox(text=word_text, coordinates=vertices)
                        )

        response = OCRResponse(
            all_text=extracted_text.strip(), bounding_boxes=bounding_boxes
        )
        return response

    def get_name(self) -> str:
        return "Google Vision - OCR"

    @classmethod
    def init_from_env(cls) -> "GCPOCR":
        credentials_path = os.environ.get("GOOGLE_APPLICATION_CREDENTIALS")
        if not credentials_path:
            raise EnvironmentError(
                "GOOGLE_APPLICATION_CREDENTIALS environment variable is not set"
            )
        credentials = service_account.Credentials.from_service_account_file(
            credentials_path
        )
        return cls(credentials)


# factory


def gcp_provider_factory(task: Task, **config) -> ProviderHandler:
    if task not in GCP_SUPPORTED_TASKS:
        raise ValueError(f"Unsupported task: {task}")
    elif task == Task.SENTIMENT_ANALYSIS:
        return GCPSentimentAnalysis.init_from_env()
    elif task == Task.TRANSLATION:
        source_language = config.get("source_language", "en")
        target_language = config.get("target_language", "es")
        return GCPTranslation.init_from_env(
            source_language=source_language,
            target_language=target_language,
        )
    elif task == Task.OCR:
        return GCPOCR.init_from_env()
    else:
        raise ValueError(f"Unsupported task: {task}")



import boto3
import logging
import threading
import time

from datetime import UTC, datetime, timedelta
from functools import wraps
from typing import Any, Callable

from supercontrast.provider import Provider
from supercontrast.task import Task


def track_cost(func: Callable[..., Any]) -> Callable[..., Any]:
    @wraps(func)
    def wrapper(*args, **kwargs):
        if args and isinstance(args[0], Task) and isinstance(args[1], Provider):
            task = args[0]
            provider = args[1]
            if provider == Provider.AWS:
                start_time = datetime.now(UTC)
                last_month_start = start_time - timedelta(days=30)
                last_month_end = start_time

                # Get initial cost for the last month
                initial_cost = track_aws_cost(task, last_month_start, last_month_end)
                logging.info(f"Initial cost for the last month: ${initial_cost:.6f}")
                result = func(*args, **kwargs)

                # Set up hourly cost tracking
                def track_hourly_cost():
                    while True:
                        time.sleep(3600)  # Wait for an hour
                        current_time = datetime.now(UTC)
                        hourly_cost = track_aws_cost(task, start_time, current_time)
                        logging.info(
                            f"Cost incurred in the last hour: ${hourly_cost:.6f}"
                        )

                # Start hourly cost tracking in a separate thread
                cost_thread = threading.Thread(target=track_hourly_cost, daemon=True)
                cost_thread.start()

                return result
        return func(*args, **kwargs)

    return wrapper


def track_aws_cost(task: Task, start_time: datetime, end_time: datetime):
    if task == Task.OCR:
        service_name = "AmazonTextract"
    elif task == Task.TRANSLATION:
        service_name = "AmazonTranslate"
    elif task == Task.SENTIMENT_ANALYSIS:
        service_name = "AmazonComprehend"
    else:
        raise ValueError(f"Unsupported task: {task}")

    # Create a Cost Explorer client
    ce_client = boto3.client("ce", region_name="us-east-1")

    # Get cost and usage data
    response = ce_client.get_cost_and_usage(
        TimePeriod={"Start": start_time.isoformat(), "End": end_time.isoformat()},
        Granularity="HOURLY",
        Metrics=["UnblendedCost", "UsageQuantity"],
        GroupBy=[
            {"Type": "DIMENSION", "Key": "SERVICE"},
        ],
        Filter={"Dimensions": {"Key": "SERVICE", "Values": []}},
    )

    # Process the results
    for result in response["ResultsByTime"]:
        for group in result["Groups"]:
            cost = float(group["Metrics"]["UnblendedCost"]["Amount"])
            usage = float(group["Metrics"]["UsageQuantity"]["Amount"])

            if usage > 0:
                avg_cost_per_request = cost / usage
                logging.info(
                    f"Average cost per request for {task} in {service_name}: ${avg_cost_per_request:.6f}"
                )
            else:
                logging.info(
                    f"No usage recorded for {task} in {service_name} during this period."
                )

    # If no results, log a message
    if not response["ResultsByTime"]:
        logging.info(
            f"No cost data available for {task} in {service_name} during this period."
        )



import re
import unicodedata

from num2words import num2words


def truncate_text(text: str, max_bytes: int = 5000):
    """Truncate text to a maximum of max_bytes bytes."""
    return text.encode("utf-8")[:max_bytes].decode("utf-8", "ignore")


def normalize_text(text: str, task: str = "transcription") -> str:
    """Normalize text for comparison."""
    # Convert to lowercase
    text = text.lower()

    # Unicode normalization (convert to standard form)
    text = unicodedata.normalize("NFKD", text)

    # Remove diacritical marks
    text = "".join(c for c in text if not unicodedata.combining(c))

    # Replace numbers with their word equivalents
    def replace_number(match):
        number = match.group()
        try:
            return num2words(int(number))
        except ValueError:
            return num2words(float(number))

    text = re.sub(r"\b\d+(?:\.\d+)?\b", replace_number, text)

    # Remove punctuation and special characters
    text = re.sub(r"[^\w\s]", " ", text)

    # Normalize whitespace
    text = " ".join(text.split())

    return text



from supercontrast.optimizer.optimizer_handler import OptimizerHandler
from supercontrast.provider.provider_enum import Provider


class OptimizerMockHandler(OptimizerHandler):
    def get_provider(self) -> Provider:
        if not self.providers:
            raise ValueError("No providers available")
        return self.providers[0]



from pydantic import BaseModel
from typing import List, Tuple, Union

# Request


class OCRRequest(BaseModel):
    image: Union[str, bytes]


# Response


class OCRBoundingBox(BaseModel):
    text: str
    coordinates: List[Tuple[int, int]]

    def __str__(self):
        return f"OCRBoundingBox(text: '{self.text}', coordinates: {self.coordinates})"


class OCRResponse(BaseModel):
    all_text: str
    bounding_boxes: List[OCRBoundingBox]

    def __str__(self):
        bounding_boxes_str = "\n    ".join(str(box) for box in self.bounding_boxes)
        return f"OCRResponse(\n  all_text: '{self.all_text}',\n  bounding_boxes: [\n    {bounding_boxes_str}\n  ]\n)"



import nltk

from nltk.tokenize import word_tokenize
from nltk.translate.bleu_score import sentence_bleu as nltk_sentence_bleu
from sacrebleu import sentence_bleu

from supercontrast.metrics.metrics_calculator import MetricsCalculator
from supercontrast.metrics.metrics_enum import Metric


class BLEUCalculator(MetricsCalculator[str, float]):
    def __init__(self, *args, **kwargs):
        super().__init__(Metric.BLEU, *args, **kwargs)

    def calculate(self, reference: str, hypothesis: str) -> float:
        return sentence_bleu(hypothesis, [reference]).score

    def get_name(self) -> str:
        return "BLEU Score"


class BLEUNLTKCalculator(MetricsCalculator[str, float]):
    def __init__(self, *args, **kwargs):
        super().__init__(Metric.BLEU_NLTK, *args, **kwargs)
        nltk.download("punkt", quiet=True)

    def calculate(self, reference: str, hypothesis: str) -> float:
        reference_tokens = word_tokenize(reference)
        hypothesis_tokens = word_tokenize(hypothesis)
        bleu_score = nltk_sentence_bleu([reference_tokens], hypothesis_tokens)
        if isinstance(bleu_score, float):
            return bleu_score
        if isinstance(bleu_score, int):
            return float(bleu_score)
        else:
            raise ValueError(f"Invalid result type: {type(bleu_score)}")

    def get_name(self) -> str:
        return "BLEU Score (NLTK)"



from abc import ABC, abstractmethod
from typing import Any, Dict

from supercontrast.metrics.calculators.bleu_calculator import (
    BLEUCalculator,
    BLEUNLTKCalculator,
)
from supercontrast.metrics.calculators.character_calculator import (
    CharacterErrorRateCalculator,
)
from supercontrast.metrics.calculators.chrf_calculator import CHRFCalculator
from supercontrast.metrics.calculators.meteor_calculator import METEORCalculator
from supercontrast.metrics.calculators.word_calculator import (
    MatchErrorRateCalculator,
    WordErrorRateCalculator,
    WordInformationLostCalculator,
    WordInformationPreservedCalculator,
    WordRecognitionRateCalculator,
)
from supercontrast.metrics.metrics_calculator import MetricsCalculator
from supercontrast.metrics.metrics_enum import Metric
from supercontrast.metrics.metrics_response import MetricsResponse
from supercontrast.task.types.ocr_types import OCRResponse
from supercontrast.task.types.transcription_types import TranscriptionResponse
from supercontrast.task.types.translation_types import TranslationResponse
from supercontrast.utils.text import normalize_text

# get metrics calculator


def get_metrics_calculator(metric: Metric, **config) -> MetricsCalculator:
    if metric == Metric.BLEU:
        return BLEUCalculator(**config)
    elif metric == Metric.BLEU_NLTK:
        return BLEUNLTKCalculator(**config)
    elif metric == Metric.CER:
        return CharacterErrorRateCalculator(**config)
    elif metric == Metric.WER:
        return WordErrorRateCalculator(**config)
    elif metric == Metric.MER:
        return MatchErrorRateCalculator(**config)
    elif metric == Metric.WIL:
        return WordInformationLostCalculator(**config)
    elif metric == Metric.WIP:
        return WordInformationPreservedCalculator(**config)
    elif metric == Metric.WRR:
        return WordRecognitionRateCalculator(**config)
    elif metric == Metric.METEOR:
        return METEORCalculator(**config)
    elif metric == Metric.CHRF:
        return CHRFCalculator(**config)
    else:
        raise ValueError(f"Unsupported metric: {metric}")


# metrics handler


class MetricsHandler(ABC):
    def __init__(self):
        self.metrics_calculators = self.get_metrics_calculators()

    @abstractmethod
    def get_metrics_calculators(self) -> Dict[Metric, MetricsCalculator]:
        pass

    @abstractmethod
    def calculate_metrics(self, reference: Any, prediction: Any) -> MetricsResponse:
        pass


class OCRMetricsHandler(MetricsHandler):
    def get_metrics_calculators(self) -> Dict[Metric, MetricsCalculator]:
        return {
            Metric.CER: CharacterErrorRateCalculator(),
            Metric.WER: WordErrorRateCalculator(),
        }

    def calculate_metrics(
        self, reference: OCRResponse, prediction: OCRResponse
    ) -> MetricsResponse:
        normalized_reference = normalize_text(reference.all_text, "ocr")
        normalized_prediction = normalize_text(prediction.all_text, "ocr")

        metrics: Dict[Metric, Any] = {}
        for metric, calculator in self.metrics_calculators.items():
            metrics[metric] = calculator.calculate(
                normalized_reference, normalized_prediction
            )
        return MetricsResponse(
            metrics=metrics,
            normalized_reference=normalized_reference,
            normalized_prediction=normalized_prediction,
        )


class TranscriptionMetricsHandler(MetricsHandler):
    def get_metrics_calculators(self) -> Dict[Metric, MetricsCalculator]:
        return {
            Metric.WER: WordErrorRateCalculator(),
            Metric.MER: MatchErrorRateCalculator(),
            Metric.WIL: WordInformationLostCalculator(),
            Metric.CER: CharacterErrorRateCalculator(),
            Metric.WIP: WordInformationPreservedCalculator(),
            Metric.WRR: WordRecognitionRateCalculator(),
        }

    def calculate_metrics(
        self, reference: TranscriptionResponse, prediction: TranscriptionResponse
    ) -> MetricsResponse:
        normalized_reference = normalize_text(reference.text, "transcription")
        normalized_prediction = normalize_text(prediction.text, "transcription")

        metrics = {}
        for metric, calculator in self.metrics_calculators.items():
            metrics[metric] = calculator.calculate(
                normalized_reference, normalized_prediction
            )
        return MetricsResponse(
            metrics=metrics,
            normalized_reference=normalized_reference,
            normalized_prediction=normalized_prediction,
        )


class TranslationMetricsHandler(MetricsHandler):
    def get_metrics_calculators(self) -> Dict[Metric, MetricsCalculator]:
        return {
            Metric.BLEU: BLEUCalculator(),
            Metric.BLEU_NLTK: BLEUNLTKCalculator(),
            Metric.METEOR: METEORCalculator(),
            Metric.CHRF: CHRFCalculator(),
        }

    def calculate_metrics(
        self, reference: TranslationResponse, prediction: TranslationResponse
    ) -> MetricsResponse:
        normalized_reference = normalize_text(reference.text, "translation")
        normalized_prediction = normalize_text(prediction.text, "translation")

        metrics = {}
        for metric, calculator in self.metrics_calculators.items():
            metrics[metric] = calculator.calculate(
                normalized_reference, normalized_prediction
            )
        return MetricsResponse(
            metrics=metrics,
            normalized_reference=normalized_reference,
            normalized_prediction=normalized_prediction,
        )



from test_utils import print_request_response_and_metadata

from supercontrast import (
    Provider,
    SuperContrastClient,
    Task,
    TaskMetadata,
    TranscriptionRequest,
    TranscriptionResponse,
    get_supported_providers_for_task,
)

# constants

TEST_AUDIO_URL = "https://github.com/supercontrast-ai/supercontrast/raw/main/tests/audio/test_transcription.wav"
REFERENCE_TRANSCRIPTION = "November 10th, Wednesday, 9:00 PM. I'm standing in a dark alley. After waiting several hours, the time has come. A woman with long dark hair approaches. I have to act, and fast, before she realizes what has happened. I must find out."

# tests


def test_transcription_azure():
    transcription_azure_client = SuperContrastClient(
        task=Task.TRANSCRIPTION, providers=[Provider.AZURE]
    )
    request = TranscriptionRequest(audio_file=TEST_AUDIO_URL)
    reference_response = TranscriptionResponse(text=REFERENCE_TRANSCRIPTION)
    response, metadata = transcription_azure_client.request(
        request, reference=reference_response
    )

    assert response is not None
    assert isinstance(response.text, str)
    assert len(response.text) > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.TRANSCRIPTION
    assert metadata.provider == Provider.AZURE
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.TRANSCRIPTION, request, response, metadata)


def test_transcription_openai():
    transcription_openai_client = SuperContrastClient(
        task=Task.TRANSCRIPTION, providers=[Provider.OPENAI]
    )
    request = TranscriptionRequest(audio_file=TEST_AUDIO_URL)
    reference_response = TranscriptionResponse(text=REFERENCE_TRANSCRIPTION)
    response, metadata = transcription_openai_client.request(
        request, reference=reference_response
    )

    assert response is not None
    assert isinstance(response.text, str)
    assert len(response.text) > 0

    assert metadata is not None
    assert isinstance(metadata, TaskMetadata)
    assert metadata.task == Task.TRANSCRIPTION
    assert metadata.provider == Provider.OPENAI
    assert metadata.latency > 0

    print_request_response_and_metadata(Task.TRANSCRIPTION, request, response, metadata)


# evaluate


def test_transcription_evaluate():
    transcription_client = SuperContrastClient(
        task=Task.TRANSCRIPTION, providers=get_supported_providers_for_task(Task.TRANSCRIPTION)
    )
    request = TranscriptionRequest(audio_file=TEST_AUDIO_URL)
    reference_response = TranscriptionResponse(text=REFERENCE_TRANSCRIPTION)
    responses = transcription_client.evaluate(request, reference=reference_response)

    assert responses is not None
    assert isinstance(responses, dict)
    assert all(
        isinstance(response, tuple)
        and len(response) == 2
        and all(
            isinstance(item, (TranscriptionResponse, TaskMetadata)) for item in response
        )
        for response in responses.values()
    )

    for _, (response, metadata) in responses.items():
        print_request_response_and_metadata(
            Task.TRANSCRIPTION, request, response, metadata
        )



from setuptools import find_packages, setup

with open("requirements.txt") as f:
    required = [line.strip() for line in f if line.strip() and not line.startswith("#")]

setup(
    name="supercontrast",
    version="0.0.2",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    install_requires=required,
    author="supercontrast",
    author_email="shravan@supercontrast.com",
    description="supercontrast is a package for unifying machine learning models across providers",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/supercontrast-ai/supercontrast",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.10",
)



from supercontrast.client import SuperContrastClient
from supercontrast.metrics.metrics_enum import Metric
from supercontrast.optimizer.optimizer_enum import Optimizer
from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_factory import get_supported_tasks_for_provider
from supercontrast.task.task_enum import Task
from supercontrast.task.task_factory import get_supported_providers_for_task
from supercontrast.task.task_handler import TaskHandler
from supercontrast.task.task_metadata import TaskMetadata
from supercontrast.task.types.ocr_types import OCRRequest, OCRResponse
from supercontrast.task.types.sentiment_analysis_types import (
    SentimentAnalysisRequest,
    SentimentAnalysisResponse,
)
from supercontrast.task.types.transcription_types import (
    TranscriptionRequest,
    TranscriptionResponse,
)
from supercontrast.task.types.translation_types import (
    TranslationRequest,
    TranslationResponse,
)



import os
import pydantic

from langchain.output_parsers import OutputFixingParser, PydanticOutputParser
from langchain.prompts import PromptTemplate
from langchain_anthropic import ChatAnthropic

from supercontrast.provider.provider_enum import Provider
from supercontrast.provider.provider_handler import ProviderHandler
from supercontrast.task.task_enum import Task
from supercontrast.task.types.sentiment_analysis_types import (
    SentimentAnalysisRequest,
    SentimentAnalysisResponse,
)
from supercontrast.task.types.translation_types import (
    TranslationRequest,
    TranslationResponse,
)
from supercontrast.utils.text import truncate_text

# Constants

ANTHROPIC_MODEL_NAME = "claude-3-5-sonnet-20240620"
ANTHROPIC_SUPPORTED_TASKS = [Task.SENTIMENT_ANALYSIS, Task.TRANSLATION]

# Task.SENTIMENT_ANALYSIS


class SentimentAnalysisOutput(pydantic.BaseModel):
    score: float = pydantic.Field(
        description="Sentiment score between -1 (very negative) and 1 (very positive)"
    )


SENTIMENT_ANALYSIS_PROMPT = PromptTemplate(
    template="Analyze the sentiment of the following text. Respond with 1 for positive, 0 for neutral, and -1 for negative:\n\n{text}\n\n{format_instructions}",
    input_variables=["text"],
    partial_variables={
        "format_instructions": PydanticOutputParser(
            pydantic_object=SentimentAnalysisOutput
        ).get_format_instructions()
    },
)


class AnthropicSentimentAnalysis(ProviderHandler):
    def __init__(self):
        super().__init__(provider=Provider.ANTHROPIC, task=Task.SENTIMENT_ANALYSIS)
        model = ChatAnthropic(
            **{
                "temperature": 0,
                "model": ANTHROPIC_MODEL_NAME,
            }
        )
        pydantic_parser = PydanticOutputParser(pydantic_object=SentimentAnalysisOutput)
        self.parser = OutputFixingParser.from_llm(parser=pydantic_parser, llm=model)
        self.generator = SENTIMENT_ANALYSIS_PROMPT | model | self.parser

    def request(self, request: SentimentAnalysisRequest) -> SentimentAnalysisResponse:
        result: SentimentAnalysisOutput = self.generator.invoke(
            {"text": truncate_text(request.text)}
        )
        return SentimentAnalysisResponse(score=result.score)

    def get_name(self) -> str:
        return "Anthropic - Sentiment Analysis"

    @classmethod
    def init_from_env(cls) -> "AnthropicSentimentAnalysis":
        api_key = os.environ.get("ANTHROPIC_API_KEY")
        if not api_key:
            raise ValueError("ANTHROPIC_API_KEY environment variable is not set")
        return cls()


# Task.TRANSLATION


class TranslationOutput(pydantic.BaseModel):
    translation: str = pydantic.Field(description="Translated text")


TRANSLATION_PROMPT = PromptTemplate(
    template="Translate the following text from {src_language} to {target_language}:\n\n{text}\n\n{format_instructions}",
    input_variables=["src_language", "target_language", "text"],
    partial_variables={
        "format_instructions": PydanticOutputParser(
            pydantic_object=TranslationOutput
        ).get_format_instructions()
    },
)


class AnthropicTranslate(ProviderHandler):
    def __init__(self, src_language: str, target_language: str):
        super().__init__(provider=Provider.ANTHROPIC, task=Task.TRANSLATION)
        model = ChatAnthropic(
            **{
                "temperature": 0,
                "model": ANTHROPIC_MODEL_NAME,
            }
        )
        pydantic_parser = PydanticOutputParser(pydantic_object=TranslationOutput)
        self.parser = OutputFixingParser.from_llm(parser=pydantic_parser, llm=model)
        self.generator = TRANSLATION_PROMPT | model | self.parser
        self.src_language = src_language
        self.target_language = target_language

    def request(self, request: TranslationRequest) -> TranslationResponse:
        result: TranslationOutput = self.generator.invoke(
            {
                "src_language": self.src_language,
                "target_language": self.target_language,
                "text": truncate_text(request.text),
            }
        )
        return TranslationResponse(text=result.translation)

    def get_name(self) -> str:
        return "Anthropic - Translate"

    @classmethod
    def init_from_env(
        cls, source_language: str, target_language: str
    ) -> "AnthropicTranslate":
        api_key = os.environ.get("ANTHROPIC_API_KEY")
        if not api_key:
            raise ValueError("ANTHROPIC_API_KEY environment variable is not set")
        return cls(source_language, target_language)


# factory


def anthropic_provider_factory(task: Task, **config) -> ProviderHandler:
    if task not in ANTHROPIC_SUPPORTED_TASKS:
        raise ValueError(f"Unsupported task: {task}")
    elif task == Task.SENTIMENT_ANALYSIS:
        return AnthropicSentimentAnalysis.init_from_env()
    elif task == Task.TRANSLATION:
        source_language = config.get("source_language", "en")
        target_language = config.get("target_language", "es")
        return AnthropicTranslate.init_from_env(source_language, target_language)
    else:
        raise ValueError(f"Unsupported task: {task}")



from enum import Enum


class Metric(Enum):
    # Word-level metrics
    WER = "word_error_rate"
    MER = "match_error_rate"
    WIL = "word_information_lost"
    WIP = "word_information_preserved"
    WRR = "word_recognition_rate"

    # Character-level metric
    CER = "character_error_rate"

    # Translation-specific metrics
    BLEU = "bilingual_evaluation_understudy"
    BLEU_NLTK = "bleu_nltk"
    METEOR = "metric_for_evaluation_of_translation_with_explicit_ordering"
    CHRF = "character_n_gram_f_score"



import os
import requests
import tempfile


def load_audio_file(audio_file: str) -> str:
    if audio_file.startswith(("http://", "https://")):
        response = requests.get(audio_file)
        response.raise_for_status()

        # Extract the file extension from the URL
        file_extension = os.path.splitext(audio_file)[1]

        with tempfile.NamedTemporaryFile(
            delete=False, suffix=file_extension
        ) as temp_file:
            temp_file.write(response.content)
            temp_file_path = temp_file.name

        return temp_file_path
    else:
        return audio_file



from typing import Optional

from supercontrast.metrics.metrics_handler import (
    MetricsHandler,
    OCRMetricsHandler,
    TranscriptionMetricsHandler,
    TranslationMetricsHandler,
)
from supercontrast.task.task_enum import Task


# metrics factory
def metrics_factory(task: Task) -> Optional[MetricsHandler]:
    if task == Task.OCR:
        return OCRMetricsHandler()
    elif task == Task.TRANSCRIPTION:
        return TranscriptionMetricsHandler()
    elif task == Task.TRANSLATION:
        return TranslationMetricsHandler()
    elif task == Task.SENTIMENT_ANALYSIS:
        return None
    else:
        raise ValueError(f"Unsupported task: {task}")



from pydantic import BaseModel
from typing import Any, Dict, Generic, Optional, TypeVar

from supercontrast.metrics.metrics_enum import Metric
from supercontrast.provider.provider_enum import Provider
from supercontrast.task.task_enum import Task

ResponseType = TypeVar("ResponseType")


class TaskMetadata(BaseModel, Generic[ResponseType]):
    task: Task
    provider: Provider
    latency: float
    reference: Optional[ResponseType] = None
    normalized_reference: Optional[ResponseType] = None
    normalized_prediction: Optional[ResponseType] = None
    metrics: Optional[Dict[Metric, Any]] = None

    def __str__(self):
        lines = [
            f"Task: {self.task}",
            f"Provider: {self.provider}",
            f"Latency: {self.latency:.2f}s",
        ]
        if self.reference:
            lines.append(f"Reference: {self.reference}")
        if self.normalized_reference:
            lines.append(f"Normalized Reference: {self.normalized_reference}")
        if self.normalized_prediction:
            lines.append(f"Normalized Prediction: {self.normalized_prediction}")
        if self.metrics:
            lines.append("Metrics:")
            for metric, value in self.metrics.items():
                lines.append(f"  {metric}: {value}")
        return "\n".join(lines)



from typing import List, Optional

from supercontrast.optimizer.handlers.mock import OptimizerMockHandler
from supercontrast.optimizer.optimizer_enum import Optimizer
from supercontrast.optimizer.optimizer_handler import OptimizerHandler
from supercontrast.provider.provider_enum import Provider
from supercontrast.task.task_enum import Task


def optimizer_factory(
    task: Task,
    providers: List[Provider],
    optimizer: Optional[Optimizer] = None,
) -> OptimizerHandler:
    if optimizer is None:
        return OptimizerMockHandler(task, providers)
    elif optimizer == Optimizer.LATENCY:
        return OptimizerMockHandler(task, providers)
    elif optimizer == Optimizer.COST:
        return OptimizerMockHandler(task, providers)
    else:
        raise ValueError(f"Unsupported optimizer: {optimizer}")



from typing import List, Optional

from supercontrast.optimizer.optimizer_enum import Optimizer
from supercontrast.provider.provider_enum import Provider
from supercontrast.task.task_enum import Task
from supercontrast.task.task_factory import task_factory
from supercontrast.task.task_handler import TaskHandler


class SuperContrastClient:
    def __init__(
        self,
        task: Task,
        providers: List[Provider],
        optimizer: Optional[Optimizer] = None,
        **config
    ):
        self.task_handler: TaskHandler = task_factory(
            task, providers, optimizer, **config
        )

    def request(self, *args, **kwargs):
        return self.task_handler.request(*args, **kwargs)

    def evaluate(self, *args, **kwargs):
        return self.task_handler.evaluate(*args, **kwargs)
